/*
 **********************************************************************************************************************************************
 *  Copyright 2005/2006, Pascal Pfiffner.                                                                                                     *
 *                                                                                                                                            *
 *  This Widget is Freeware and will always be. However, if you`d like to use some of its code, please tell me at "help.you.i.can@gmail.com"  *
 **********************************************************************************************************************************************
 */

// Globals
var run = true;		// set to false to stop the animation
var mode = "clock";		// "clock" or "game"
var scoreLeft = 0;
var scoreRight = 0;

// Globals for Apple
var flipShown = false;
var animation = { timer:null, duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, secondElement:null };


function showVersion() {
	getObj("infospan").innerHTML = "v 0.2";
}


// Dashboard Setup
function setup() {
	// widget-events
	if(window.widget) {
		widget.onshow = onShow;
		widget.onhide = onHide;
		widget.onremove = doCleanup;
	}
	
	// images
	for(var i = 0; i < 10; i++) {
		var img = new Image();
		img.src = "./Images/" + i + ".gif";
	}
	
	// start
	createGenericButton(getObj("done"), "done", savePrefs);
	play();
}



function play() {
	run = true;
	showTime();
	startBallTimed();
}

function stop() {
	run = false;
}



var ballControl = {
				ballSize:6,
				bounceDist:18,			// distance of the bar-surface (where the ball hits) to the margin of the playfield
				playfieldWidth:108,		// width from bar to bar (inner distance bar to bar _MINUS_ the ball-width, used for the x-value of the ball)
				fieldTotalHeight:110,	// full height
				fieldHeight:104,		// full height minus the ball size (used to give the ball an y-value)
				
				lastStart:new Date(),
				speed:1000,			// ms to perform one length of the playfield (till the player hits)
				direction:1,		// direction: 1 means "right", -1 "left"
				startAngle:30,		// positive angle means down, negative angle means up
				lastAngle:40,
				yRounded:0,			// saves how many decimals were rounded
				projectedY:30,		// which Y-value the ball will have when it arrives on the players line
				inGoal:false };		// if ball passes player-line and does not hit, do not check for hit behind playerline (set this to true)


// sets a random start-position for the ball and returns that y-value
function initializeBall() {
	var ball = getObj("playBall");
	var y = Math.round(Math.random() * ballControl.fieldHeight);		// a more or less random start-position
	ball.style.top = y + "px";
	ballControl.lastAngle = ballControl.startAngle;
	ballControl.inGoal = false;
	return y;
}



function startBallTimed() {
	if(mode != "clock")
		return;
	
	var second = (new Date()).getSeconds();
	
	// while we count the seconds, place the ball and start ballMove, which moves the ball
	if((second > 3) && (second < 59)) {
		var y = initializeBall();
		setProjectedY(y);
		
		ballControl.lastStart = (new Date()).getTime();
		ballMove();
		
		setTimeout("movePlayerAuto(" + ballControl.direction + ")", 200);
	}
		
	// one minute passed
	else {
		var milli = (new Date()).getMilliseconds();
		
		setTimeout("showTime()", 1000);
		setTimeout("startBallTimed()", (1000 * (4 - second)) - milli);		// re-calls itself to relaunch on the 4th second of the minute
	}
}


function ballMove() {
	var now = (new Date()).getTime();
	milli = now - ballControl.lastStart;						// how long did it take from the last move?
	milli = (milli == 0) ? 1 : milli;							// remove a 0-ms-offset
	var fraction = Math.abs(milli / ballControl.speed);			// get the fraction (the amount we need to move of the total playfield-width)
	var settimeout = false;
	
	var ball = getObj("playBall");
	var x = ball.style.left.replace(/[^\d\.-]/g, "");
	x = isNaN(x) ? -10 : x;
	var y = ball.style.top.replace(/[^\d\.-]/g, "");
	y = isNaN(y) ? 70 : y;
	
	// calc a new x-value
	var deltaX = Math.round(ballControl.playfieldWidth * fraction);
	if(deltaX == 0) {
		if(run)
			setTimeout("ballMove()", 40);
		return;
	}
	var nextX = (x * 1) + (ballControl.direction * deltaX);
	
	// calculate the Y-value
	var deltaY = getDeltaY(deltaX);
	var nextY = ((y * 1) + deltaY);
	
	
	// bounce the ball off the top and off the bottom
	while((nextY < 0) || (nextY > ballControl.fieldHeight)) {
		
		// bounce off top
		if(nextY < 0) {
			nextY = nextY * -1;
			ballControl.lastAngle *= -1;
		}
		
		// bounce off the bottom
		else if(nextY > ballControl.fieldHeight) {
			nextY = ballControl.fieldHeight - (nextY - ballControl.fieldHeight);
			ballControl.lastAngle *= -1;
		}
	}
	ball.style.top = nextY + "px";
	
	
	// does the ball bounce from the playerbar?
	if(ballHitsPlayer(nextX, nextY)) {
		if(mode == "clock")
			setTimeout("movePlayerAuto(" + ballControl.direction + ")", 260);
		settimeout = true;
	}
	
	// ball out of bounds
	else if(ballOutOfBounds(nextX, nextY)) {
		if(mode == "clock")
			startBallTimed();
		else
			updateScore();
	}
	
	// simply move the ball
	else {
		ball.style.left = nextX + "px";
		settimeout = true;
	}
	
	if(settimeout && run) {
		setTimeout("ballMove()", 40);
		ballControl.lastStart = now;
	}
	
	/*/ debug
	if(nextX > x && nextX > 115) {
		alert("y: " + nextY + " when x is " + nextX);
	}
	// ----- */
	//alert("deltaY: " + deltaY + ", oldY: " + y + ", newY: " + nextY + ", deltaX: " + deltaX + ", oldX: " + x + ", newX: " + nextX);
}



// calculates delta-y from delta-x and the given angle.
function getDeltaY(deltaX, no_advanced_functions) {
	var angle = ballControl.lastAngle;
	if(!deltaX || deltaX == 0)
		deltaX = 1;
	
	deltaX = Math.abs(deltaX);
	
	var deltaY = 0;
	var deltaYfull = eval(deltaX * Math.tan(ballControl.lastAngle * Math.PI / 180));
	var direction = 1;			// 1 is down (add deltaY to old Y-value)
	if(deltaYfull < 0) {
		direction = -1;
		deltaYfull = Math.abs(deltaYfull);
	}
	
	if(!no_advanced_functions)
		deltaYfull = Math.abs(deltaYfull + ballControl.yRounded);
	
	while(deltaYfull > 1) {
		deltaYfull -= 1;
		deltaY += 1;
	}
	
	if(!no_advanced_functions)
		ballControl.yRounded = deltaYfull;
	
	return deltaY * direction;
}

// sets the projected Y
function setProjectedY(y) {
	if(isNaN(y))
		return 0;
	
	var currX = getObj("playBall").style.left.replace(/[^\d\.-]/g, "");							// current X
	var deltaX = (ballControl.direction == 1) ?
						(1* ballControl.playfieldWidth + ballControl.bounceDist - currX) : 
						(currX - ballControl.bounceDist);		// distance from current ball-position to surface of the matching playerbar
	var projY = (y * 1) + getDeltaY(deltaX, true);
	while((projY < 0) || (projY > ballControl.fieldHeight)) {
		
		// top
		if(projY < 0)
			projY = projY * -1;
		
		// bottom
		else if(projY > ballControl.fieldHeight)
			projY = Math.abs(ballControl.fieldHeight - (projY - ballControl.fieldHeight));
	}
	
	//alert("setProjectedY: direction = " + ballControl.direction + ", currX = " + currX + ", deltaX = " + deltaX + ", projY = " + projY);
	ballControl.projectedY = projY;
}

// returns a bool if the player hits the ball
function ballHitsPlayer(ballX, ballY) {
	var bar = (ballControl.direction == 1) ? getObj("rightBar") : getObj("leftBar");
	var barX = (ballControl.direction == 1) ? Math.abs(ballControl.bounceDist + ballControl.playfieldWidth) : ballControl.bounceDist;
	var barY = bar.style.top.replace(/[^\d\.-]/g, "");
	
	//alert(bar + " has " + barX + " and " + barY + ", ballX is " + ballX + " and ballY is " + ballY);
	
	// check for x
	if((ballControl.direction == 1) ? (ballX >= barX) : (ballX <= barX)) {
		
		// check for y
		if(!ballControl.inGoal && (barY < ((1* ballY) + ballControl.ballSize)) && (barY > (ballY - playerMove.barHeight))) {
			
			// YES, ball hit playerbar
			(getObj("playBall")).style.left = ((ballControl.direction == 1) ? (1* ballControl.playfieldWidth + ballControl.bounceDist) : ballControl.bounceDist) + "px";
			var hitHeight = 1* playerMove.barHeight + ballControl.ballSize;
			var absY = ballY - (barY - ballControl.ballSize);
			var direction = 1;				// 1 = hit in upper part, -1 = hit in lower part of the bar
			if(absY > (hitHeight / 2)) {
				absY = hitHeight - absY;
				direction = -1;
			}
			
			// change angle according to the hit-position
			if(absY < playerMove.barMargin) {
				var deltaAngle = Math.round(playerMove.anglePercentageChange * (90 - (direction * ballControl.lastAngle)));
				var deltaHit = (absY == 0) ? 1 : (1 - (absY / playerMove.barMargin));
				deltaAngle = Math.round(deltaHit * deltaAngle * direction * -1);
				//alert("in margin: deltaAngle=" + deltaAngle + ", last=" + ballControl.lastAngle + ", new=" + (ballControl.lastAngle + (1* deltaAngle)));
				ballControl.lastAngle = ballControl.lastAngle + (1* deltaAngle);
				ballControl.lastAngle = (ballControl.lastAngle < -66) ? -66 : ((ballControl.lastAngle > 66) ? 66 : ballControl.lastAngle);
			}
			
			ballControl.direction *= -1;
			setProjectedY(ballY);
			return true;
		}
		else {
			ballControl.inGoal = true;
		}
	}
	return false;
}

function ballOutOfBounds(ballX, ballY) {
	
	// check for x only
	if((ballControl.direction == 1) && (ballX > Math.abs(2 * ballControl.bounceDist + ballControl.playfieldWidth))) {
		getObj("playBall").style.left = (ballControl.ballSize * -1) + "px";
		ballControl.direction = 1;
		return true;
	}
	else if(ballX < (ballControl.ballSize * -1)) {
		getObj("playBall").style.left = (2 * ballControl.bounceDist + ballControl.playfieldWidth + ballControl.ballSize) + "px";
		ballControl.direction = -1;
		return true;
	}
	
	return false;
}


/*
 *	Moving the Bars
 */
var playerMove = {
			leftStart:30, rightStart:40,	// start-position of the left player
			leftAim:0, rightAim:0,			// start-position of the right player
			barHeight:24,					// height of the bars
			barMargin:8,					// within this margin the angle of the ball changes its angle
			anglePercentageChange:0.15,		// on the outmost hit-point, the angel of the ball changes this much compared to his old angle
			
			starttime:0,
			endtime:0,
			speed:400
		};
function movePlayerAuto(which) {
	var bar = (which == 1) ? getObj("rightBar") : getObj("leftBar");
	
	var now = new Date();
	var new_hour = (now.getMinutes() == 59);
	var too_late = (mode == "clock" && (new Date()).getSeconds() >= 57);
	
	// lets the ball hit at a random point on the bar (5 pixel security-margin)
	var randOffset = Math.round((Math.random() * (1* playerMove.barHeight + ballControl.ballSize - 10)) - (playerMove.barHeight - 5));
	
	if(which == 1) {
		playerMove.rightStart = parseInt(bar.style.top);
		playerMove.rightAim = ballControl.projectedY + randOffset;
		if(new_hour && too_late)
			playerMove.rightAim = (playerMove.rightAim < 40) ? (playerMove.rightAim + 40) : (playerMove.rightAim - 40);		// on new hour, right player misses
	}
	else {
		playerMove.leftStart = parseInt(bar.style.top);
		playerMove.leftAim = ballControl.projectedY + randOffset;
		if(!new_hour && too_late)
			playerMove.leftAim = (playerMove.leftAim < 40) ? (playerMove.leftAim + 40) : (playerMove.leftAim - 40);			// left player misses every minute but 59th
	}
	
	playerMove.starttime = (new Date()).getTime();
	playerMove.endtime = playerMove.starttime + playerMove.speed;
	movePlayerTimed(which);
	
	//alert("moving player " + which + " to " + ((which == 1) ? playerMove.rightAim : playerMove.leftAim) + ", projectedY ist " + ballControl.projectedY);
}

function movePlayerTimed(which) {
	var bar = (which == 1) ? getObj("rightBar") : getObj("leftBar");
	var now = (new Date()).getTime();
	
	// right bar
	var start = isNaN(playerMove.rightStart) ? 40 : playerMove.rightStart;
	var aim = playerMove.rightAim;
	
	// left bar
	if(which != 1) {
		start = isNaN(playerMove.leftStart) ? 30 : playerMove.leftStart;
		aim = playerMove.leftAim;
	}
	
	// last move, go to aimed position
	if(now >= playerMove.endtime) {
		aim = (aim < 0) ?
					0
					: ((aim > Math.abs(ballControl.fieldTotalHeight - playerMove.barHeight)) ?
							Math.abs(ballControl.fieldTotalHeight - playerMove.barHeight)
							: aim
					);
		bar.style.top = aim + "px";
	}
	
	// new position
	else {
		var delta = (now - playerMove.starttime) / playerMove.speed;
		var fraction = ((delta * delta) * 3.0) - ((delta * delta * delta) * 2.0);
		var curr = Math.round(start + (fraction * (aim - start)));
		curr = (curr < 0) ?
					0
					: ((curr > Math.abs(ballControl.fieldTotalHeight - playerMove.barHeight)) ?
							Math.abs(ballControl.fieldTotalHeight - playerMove.barHeight)
							: curr
					);
		//alert("curr " + curr + ", start " + start + ", aim " + aim);
		bar.style.top = curr + "px";
		
		if(run)
			setTimeout("movePlayerTimed(" + which + ")", 60);
	}
}

function movePlayerManual(event) {
	alert("movePlayerManual: " + event.which + " by " + event.currentTarget);
}


function keyDown(event) {
	var which = (event.which == 40) ? "upR" : ((event.which == 38) ? "downR" : "pause");
	
	switch(which) {
		case "upR":
		case "downR":
		case "upL":
		case "downL":
			//movePlayerManual(event);
			break;
		
		default:
			break;
	}
}


function updateScore() {
	getObj("scoreLeft").innerHTML = 0;
	getObj("scoreRight").innerHTML = 0;
}

function showTime(hour, minute) {
	if(!hour)
		hour = (new Date()).getHours();
	if(!minute)
		minute = (new Date()).getMinutes();
	
	scoreLeft = hour;
	scoreRight = minute;
	
	displayNumber(-1, scoreLeft);
	displayNumber(1, scoreRight);
}

function displayNumber(where, number) {			// 1 means right, -1 means left
	if(! where)
		where = 1;
	if(isNaN(number))
		return;
	
	var obj = (where == 1) ? getObj("scoreRight") : getObj("scoreLeft");
	var strlen = number.toString().length;
	
	// remove old images
	while(obj.lastChild)
		obj.removeChild(obj.lastChild);
	
	// set new images
	for(var i = 0; i < strlen; i++) {
		var img = document.createElement("img");
		img.setAttribute("class", "number");
		img.src = "./Images/" + number.toString().substring(i, i + 1) + ".gif";
		obj.appendChild(img);
	}
}


function getObj(id) {
	return document.getElementById(id);
}




function toggleMode(event) {
	mode = (mode == "clock") ? "game" : "clock";
	var toggle = getObj("modeToggle");
	
	if(mode == "game") {
		document.addEventListener("keydown", keyDown, true);
		event.stopPropagation();
		event.preventDefault();
		toggle.innerHTML = "clock";
	}
	else {
		document.removeEventListener("keydown", keyDown, true);
		event.stopPropagation();
		event.preventDefault();
		toggle.innerHTML = "play!";
		play();
	}
	
	return;
}

function showPrefs() {
	var front = getObj("front");
	var back = getObj("back");
	
	//adjustPrefs();		// creates the option-menu to select an image and prechecks the random-checkbox
	showVersion();		// sets version-number
	
	if(window.widget)
		widget.prepareForTransition("ToBack");
	
	front.style.display = "none";
	back.style.display = "block";
	
	stop();
		
	if(window.widget)
		setTimeout("widget.performTransition();", 0);
}


function savePrefs() {
	hidePrefs();
}


function hidePrefs() {
	var front = getObj("front");
	var back = getObj("back");
	
	if(window.widget)
		widget.prepareForTransition("ToFront");
	
	//getObj("fliprollie").style.display = "none";	
	front.style.display = "block";
	back.style.display = "none";
	
	play();
	
	if(window.widget)
		setTimeout("widget.performTransition();", 0);
}






function onShow() {
	play();
}

function onHide() {
	stop();
}

function doCleanup() {
	
}


// ---------------------
// by Apple
// ---------------------
// 
//

function mousemove(event) {
	if(! flipShown) {
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		var starttime = (new Date).getTime() - 13;
 
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = getObj("flip");
		animation.secondElement = getObj("modeToggle");
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}


function mouseexit (event) {
	if (flipShown) {
		// fade in the info button
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}

		var starttime = (new Date).getTime() - 13;

		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = getObj("flip");
		animation.secondElement = getObj("modeToggle");
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate() {
	var T;
	var ease;
	var time = (new Date).getTime();
   

	T = limit_3(time-animation.starttime, 0, animation.duration);

	if (T >= animation.duration)
	{
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}

	animation.firstElement.style.opacity = animation.now;
	animation.secondElement.style.opacity = animation.now;
}

function limit_3 (a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
	return from + (to - from) * ease;
}

// these functions are called when the info button itself receives onmouseover and onmouseout events (by Apple)
// I modified them to inverse the colors on hover

function enterflip(event) {
	getObj("flip").style.backgroundImage = "url(/System/Library/WidgetResources/ibutton/white_rollie.png)";
	getObj("flippi").src = "/System/Library/WidgetResources/ibutton/black_i.png";
}

function exitflip(event) {
	getObj("flip").style.backgroundImage = "url(/System/Library/WidgetResources/ibutton/black_rollie.png)";
	getObj("flippi").src = "/System/Library/WidgetResources/ibutton/white_i.png";
}